@@ -573,6 +573,8 @@ my %epsg_vertcs = (
             3053 => 'Hjorsey 1955 Lambert',
             3057 => 'ISN93 Lambert 1993',
             3300 => 'Estonian Coordinate System of 1992',
+            3786 => 'Popular Visualisation CRS / Mercator', #PH (NC)
+            3857 => 'WGS 84 / Pseudo-Mercator', #PH (NC)
             20137 => 'Adindan UTM zone 37N',
             20138 => 'Adindan UTM zone 38N',
             20248 => 'AGD66 AMG zone 48',
